# coding: utf-8

__doc__ = "模块的 import 使用名与模块的发布名对照表"

PKGNAME_MAP = {
    "IPython": "ipython",
    "OpenSSL": "pyOpenSSL",
    "PIL": "Pillow",
    "_argon2_cffi_bindings": "argon2-cffi-bindings",
    "_cffi_backend": "cffi",
    "_distutils_hack": "setuptools",
    "_plotly_future_": "plotly",
    "_plotly_utils": "plotly",
    "_pyrsistent_version": "pyrsistent",
    "_pytest": "pytest",
    "_win32sysloader": "pywin32",
    "_winxptheme": "pywin32",
    "_yaml": "PyYAML",
    "adodbapi": "pywin32",
    "afxres": "pywin32",
    "anaconda_navigator": "anaconda-navigator",
    "anaconda_project": "anaconda-project",
    "async_timeout": "async-timeout",
    "attr": "attrs",
    "automat": "Automat",
    "babel": "Babel",
    "backports": "backports.functools-lru-cache",
    "binstar_client": "anaconda-client",
    "bottleneck": "Bottleneck",
    "brotli": "brotlipy",
    "bs4": "beautifulsoup4",
    "charset_normalizer": "charset-normalizer",
    "commctrl": "pywin32",
    "conda_build": "conda-build",
    "conda_content_trust": "conda-content-trust",
    "conda_env": "conda",
    "conda_pack": "conda-pack",
    "conda_package_handling": "conda-package-handling",
    "conda_token": "conda-token",
    "conda_verify": "conda-verify",
    "cv2": "opencv-python",
    "cython": "Cython",
    "dateutil": "python-dateutil",
    "dbi": "pywin32",
    "dde": "pywin32",
    "diff_match_patch": "diff-match-patch",
    "doc": "scikit-image",
    "erfa": "pyerfa",
    "et_xmlfile": "et-xmlfile",
    "flask": "Flask",
    "fontTools": "fonttools",
    "google": "googleapis-common-protos",
    "google_crc32c": "google-crc32c",
    "grpc": "grpcio",
    "hamcrest": "PyHamcrest",
    "heapdict": "HeapDict",
    "importlib_metadata": "importlib-metadata",
    "include": "pywin32",
    "ipykernel_launcher": "ipykernel",
    "ipython_genutils": "ipython-genutils",
    "isapi": "pywin32",
    "isympy": "sympy",
    "jinja2": "Jinja2",
    "jinja2_time": "jinja2-time",
    "jupyter": "jupyter-core",
    "jupyter_client": "jupyter-client",
    "jupyter_console": "jupyter-console",
    "jupyter_core": "jupyter-core",
    "jupyter_server": "jupyter-server",
    "jupyterlab_plotly": "plotly",
    "jupyterlab_pygments": "jupyterlab-pygments",
    "jupyterlab_server": "jupyterlab-server",
    "jupyterlab_widgets": "jupyterlab-widgets",
    "jwt": "PyJWT",
    "lazy_object_proxy": "lazy-object-proxy",
    "lib": "pywin32",
    "libarchive": "libarchive-c",
    "libfuturize": "future",
    "libpasteurize": "future",
    "libs": "pywin32",
    "markdown": "Markdown",
    "markupsafe": "MarkupSafe",
    "matplotlib_inline": "matplotlib-inline",
    "mkl": "mkl-service",
    "mkl_fft": "mkl-fft",
    "mkl_random": "mkl-random",
    "mmapfile": "pywin32",
    "mmsystem": "pywin32",
    "mpl_toolkits": "matplotlib",
    "mypy_extensions": "mypy-extensions",
    "nacl": "PyNaCl",
    "navigator_updater": "navigator-updater",
    "nest_asyncio": "nest-asyncio",
    "netbios": "pywin32",
    "ntsecuritycon": "pywin32",
    "numbergen": "param",
    "odbc": "pywin32",
    "onedal": "daal4py",
    "past": "future",
    "perfmon": "pywin32",
    "pkg_resources": "setuptools",
    "prometheus_client": "prometheus-client",
    "prompt_toolkit": "prompt-toolkit",
    "protego": "Protego",
    "pure_eval": "pure-eval",
    "pvectorc": "pyrsistent",
    "pyasn1_modules": "pyasn1-modules",
    "pydispatch": "PyDispatcher",
    "pygments": "Pygments",
    "pylab": "matplotlib",
    "pyls_spyder": "pyls-spyder",
    "pylsp": "python-lsp-server",
    "pylsp_black": "python-lsp-black",
    "pylsp_jsonrpc": "python-lsp-jsonrpc",
    "pythoncom": "pywin32",
    "pythonwin": "pywin32",
    "pyviz_comms": "pyviz-comms",
    "pywin": "pywin32",
    "pywin32_bootstrap": "pywin32",
    "pywin32_testutil": "pywin32",
    "pywintypes": "pywin32",
    "pywt": "PyWavelets",
    "pyximport": "Cython",
    "qdarkstyle": "QDarkStyle",
    "qtawesome": "QtAwesome",
    "qtpy": "QtPy",
    "rasutil": "pywin32",
    "regcheck": "pywin32",
    "regutil": "pywin32",
    "repo_cli": "conda-repo-cli",
    "requests_file": "requests-file",
    "rtree": "Rtree",
    "ruamel_yaml": "ruamel-yaml-conda",
    "run": "win-unicode-console",
    "scrapy": "Scrapy",
    "scripts": "pywin32",
    "send2trash": "Send2Trash",
    "service_identity": "service-identity",
    "servicemanager": "pywin32",
    "skimage": "scikit-image",
    "sklearn": "scikit-learn",
    "sklearnex": "scikit-learn-intelex",
    "slugify": "python-slugify",
    "smart_open": "smart-open",
    "snappy": "python-snappy",
    "socks": "PySocks",
    "sockshandler": "PySocks",
    "sphinx": "Sphinx",
    "sphinxcontrib": "sphinxcontrib-devhelp",
    "spyder_kernels": "spyder-kernels",
    "sqlalchemy": "SQLAlchemy",
    "sspi": "pywin32",
    "sspicon": "pywin32",
    "stack_data": "stack-data",
    "test": "pywin32",
    "text_unidecode": "text-unidecode",
    "three_merge": "three-merge",
    "timer": "pywin32",
    "tlz": "toolz",
    "twisted": "Twisted",
    "twisted_iocpsupport": "twisted-iocpsupport",
    "typed_ast": "typed-ast",
    "unidecode": "Unidecode",
    "websocket": "websocket-client",
    "werkzeug": "Werkzeug",
    "win2kras": "pywin32",
    "win32": "pywin32",
    "win32api": "pywin32",
    "win32clipboard": "pywin32",
    "win32com": "pywin32",
    "win32comext": "pywin32",
    "win32con": "pywin32",
    "win32console": "pywin32",
    "win32cred": "pywin32",
    "win32crypt": "pywin32",
    "win32cryptcon": "pywin32",
    "win32ctypes": "pywin32-ctypes",
    "win32event": "pywin32",
    "win32evtlog": "pywin32",
    "win32evtlogutil": "pywin32",
    "win32file": "pywin32",
    "win32gui": "pywin32",
    "win32gui_struct": "pywin32",
    "win32help": "pywin32",
    "win32inet": "pywin32",
    "win32inetcon": "pywin32",
    "win32job": "pywin32",
    "win32lz": "pywin32",
    "win32net": "pywin32",
    "win32netcon": "pywin32",
    "win32pdh": "pywin32",
    "win32pdhquery": "pywin32",
    "win32pdhutil": "pywin32",
    "win32pipe": "pywin32",
    "win32print": "pywin32",
    "win32process": "pywin32",
    "win32profile": "pywin32",
    "win32ras": "pywin32",
    "win32rcparser": "pywin32",
    "win32security": "pywin32",
    "win32service": "pywin32",
    "win32serviceutil": "pywin32",
    "win32timezone": "pywin32",
    "win32trace": "pywin32",
    "win32traceutil": "pywin32",
    "win32transaction": "pywin32",
    "win32ts": "pywin32",
    "win32ui": "pywin32",
    "win32uiole": "pywin32",
    "win32verstamp": "pywin32",
    "win32wnet": "pywin32",
    "win_inet_pton": "win-inet-pton",
    "win_unicode_console": "win-unicode-console",
    "winerror": "pywin32",
    "winioctlcon": "pywin32",
    "winnt": "pywin32",
    "winperf": "pywin32",
    "winxpgui": "pywin32",
    "winxptheme": "pywin32",
    "xlsxwriter": "XlsxWriter",
    "yaml": "PyYAML",
    "yapftests": "yapf",
    "zmq": "pyzmq",
    "zope": "zope.interface",
}
